package com.lightinit.hsdataplatform.service.admin;


import com.lightinit.hsdataplatform.entity.*;
import com.lightinit.hsdataplatform.model.ResultState;
import com.lightinit.hsdataplatform.model.admin.APIDdmandOpenReleaseModel;
import com.lightinit.hsdataplatform.model.admin.DataSupplyApiInputModel;
import com.lightinit.hsdataplatform.model.admin.DataSupplySearchModel;

import java.util.List;

public interface IDataSupplyApiService {

    RequirementApi QueryRequirementApi(long id);

    Requirement QueryRequirement(long id);

    List<Requirement> QueryRequirementList(DataSupplyApiInputModel inputModel,BaseExample.Page pager);


    long QueryRequirementListCount(DataSupplyApiInputModel inputModel);

    Integer UpDateRequirement(Requirement requirement);

    ResourceCatelog QueryCateType(long id);

    Admin QueryAdmin(long id);
    Admin QueryAdminByUserName(String username);

    ResultState release(APIDdmandOpenReleaseModel releaseModel, Long id);
}
